//  PrintDialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include "PrintDialog.h"
#include <Translate.h>
#include <UFreeType.h>
#include <UPrint.h>

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <stdiostream.h>


PrintDialog::PrintDialog (QWidget* parent, Config* configIn,
	QUWidget* uwidgetIn) : 
	QDialog (parent, "Yudit: Print", TRUE)
{
	int		i;
	const char*	fName;

	resize (275, 400);
	setCaption ("Yudit: Print");

	topConfig = configIn;
	// Title
	layout = new QGridLayout (this, 8, 2, 10);
	layout->setRowStretch (0, 2);
	layout->setRowStretch (1, 2);
	layout->setRowStretch (2, 2);
	layout->setRowStretch (3, 2);
	layout->setRowStretch (4, 2);
	layout->setRowStretch (5, 2);
	layout->setRowStretch (6, 2);
	layout->setRowStretch (7, 10);
	uLabel = new QLabel (this, "Print");
	uLabel->setAlignment (AlignHCenter|AlignVCenter);
	uLabel->setFrameStyle (QFrame::Box|QFrame::Sunken);
	uLabel->setMinimumSize (uLabel->sizeHint().width()+10,
		uLabel->sizeHint().height()*2);

	layout->addMultiCellWidget (uLabel, 0, 0, 0, 1, AlignHCenter|AlignTop);

	// Printer
	uLabelPrinter = new QLabel (this, "PrinterLabel");
	uLabelPrinter->setText (Translate("Printer:"));
	uLabelPrinter->setAlignment (AlignRight|AlignVCenter);

	uCheckPrinter = new QComboBox (this);
	if (topConfig->getPrinter()!=0)
	{
		for (i=0; i<topConfig->getPrinterList().getSize(); i++)
		{
			uCheckPrinter->insertItem (topConfig->getPrinterList().at(i));
			if (strcmp (topConfig->getPrinter(),
				topConfig->getPrinterList().at(i)) ==0)
			{
				uCheckPrinter->setCurrentItem (i);
			}
		}
	}

	uLabelPrinter->setMinimumSize (uLabelPrinter->sizeHint());
	uCheckPrinter->setMinimumSize (uCheckPrinter->sizeHint());
	layout->addWidget (uLabelPrinter, 1, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckPrinter, 1, 1, AlignLeft|AlignVCenter);
	
	// Font name
	uLabelFontName = new QLabel (this, "FontNameLabel");
	uLabelFontName->setText (Translate("Font Name:"));
	uLabelFontName->setAlignment (AlignRight|AlignVCenter);

	uCheckFontName = new QComboBox (this);
	fontCount = 0;
	for (i=0; i<topConfig->getCheckList(Config::FONTNAME).getSize(); i++)
	{
		fName = topConfig->getCheckList(Config::FONTNAME).at(i);
		if (UGetFreeTypePrint (fName)==0) continue;
		uCheckFontName->insertItem (fName);
		if (strcmp (topConfig->getChecked(Config::FONTNAME), fName)==0)
		{
			uCheckPrinter->setCurrentItem (fontCount);
		}
		uFontList.add (fName);
		fontCount++;
	}
	uLabelFontName->setMinimumSize (uLabelFontName->sizeHint());
	uCheckFontName->setMinimumSize (uCheckFontName->sizeHint());

	layout->addWidget (uLabelFontName, 2, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckFontName, 2, 1, AlignLeft|AlignVCenter);


	// Font size
	uLabelFontSize = new QLabel (this, "FontSizeLabel");
	uLabelFontSize->setText (Translate("Font Size:"));
	uLabelFontSize->setAlignment (AlignRight|AlignVCenter);

	uCheckFontSize = new QComboBox (this);
	uAddList (uCheckFontSize, 
		topConfig->getCheckList(Config::FONTSIZE),
		(const char*) topConfig->getChecked(Config::FONTSIZE));

	uLabelFontSize->setMinimumSize (uLabelFontSize->sizeHint());
	uCheckFontSize->setMinimumSize (uCheckFontSize->sizeHint());

	layout->addWidget (uLabelFontSize, 3, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckFontSize, 3, 1, AlignLeft|AlignVCenter);
	// Font Weight
	uLabelFontWeight = new QLabel (this, "FontWeightLabel");
	uLabelFontWeight->setText (Translate("Font Weight:"));
	uLabelFontWeight->setAlignment (AlignRight|AlignVCenter);

	uFontWeightList.add ("Medium");
	uFontWeightList.add ("Bold");
	uCheckFontWeight = new QComboBox (this);
	uAddList (uCheckFontWeight, uFontWeightList,
		(const char*) topConfig->getChecked(Config::FONTWEIGHT));

	uLabelFontWeight->setMinimumSize (uLabelFontWeight->sizeHint());
	uCheckFontWeight->setMinimumSize (uCheckFontWeight->sizeHint());

	layout->addWidget (uLabelFontWeight, 4, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckFontWeight, 4, 1, AlignLeft|AlignVCenter);

	// Font Slant
	uLabelFontSlant = new QLabel (this, "FontSlantLabel");
	uLabelFontSlant->setText (Translate("Font Slant:"));
	uLabelFontSlant->setAlignment (AlignRight|AlignVCenter);

	uFontSlantList.add ("Roman");
	uFontSlantList.add ("Italic");
	uCheckFontSlant = new QComboBox (this);
	uAddList (uCheckFontSlant, uFontSlantList,
		(const char*) topConfig->getChecked(Config::FONTSLANT));
	uLabelFontSlant->setMinimumSize (uLabelFontSlant->sizeHint());
	uCheckFontSlant->setMinimumSize (uCheckFontSlant->sizeHint());

	layout->addWidget (uLabelFontSlant, 5, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckFontSlant, 5, 1, AlignLeft|AlignVCenter);

	// Tabsize
	uLabelTabSize = new QLabel (this, "TabSizeLabel");
	uLabelTabSize->setText (Translate("Tab Size:"));
	uLabelTabSize->setAlignment (AlignRight|AlignVCenter);

	uCheckTabSize = new QComboBox (this);
	uAddList (uCheckTabSize, topConfig->getCheckList(Config::TABSIZE),
		(const char*) topConfig->getChecked(Config::TABSIZE));

	uLabelTabSize->setMinimumSize (uLabelTabSize->sizeHint());
	uCheckTabSize->setMinimumSize (uCheckTabSize->sizeHint());

	layout->addWidget (uLabelTabSize, 6, 0, AlignRight|AlignVCenter);
	layout->addWidget (uCheckTabSize, 6, 1, AlignLeft|AlignVCenter);

	// Buttons
	buttonOK = new QPushButton (Translate ("Ok"), this);
	buttonOK->setMaximumSize (buttonOK->sizeHint());
	
	buttonCancel = new QPushButton (Translate ("Cancel"), this);
	buttonCancel->setMaximumSize (buttonCancel->sizeHint());

	connect (buttonOK, SIGNAL (clicked()), SLOT (accept()));
	connect (buttonCancel, SIGNAL (clicked()), SLOT (reject()));

	buttonOK->setMinimumSize (buttonOK->sizeHint());
	buttonCancel->setMinimumSize (buttonCancel->sizeHint());

	layout->addWidget (buttonOK, 7, 0, AlignHCenter|AlignBottom);
	layout->addWidget (buttonCancel, 7, 1, AlignHCenter|AlignBottom);
}
PrintDialog::~PrintDialog ()
{
	if (buttonOK) delete buttonOK;
	if (buttonCancel) delete buttonCancel;

	if (uCheckFontSlant) delete uCheckFontSlant;
	if (uLabelFontSlant) delete uLabelFontSlant;

	if (uCheckFontWeight) delete uCheckFontWeight;
	if (uLabelFontWeight) delete uLabelFontWeight;

	if (uCheckFontSize) delete uCheckFontSize;
	if (uLabelFontSize) delete uLabelFontSize;

	if (uCheckFontName) delete uCheckFontName;
	if (uLabelFontName) delete uLabelFontName;

	if (uCheckPrinter) delete uCheckPrinter;
	if (uLabelPrinter) delete uLabelPrinter;

	if (uLabel) delete uLabel;
	if (layout) delete layout;
}

PrintDialog::Button
PrintDialog::print(const UCS2* const * textIn, const int textSizesIn[], int textSizeIn, 
  const char* titleIn)
{
	const char*		printerName;
	const char*		printerCommand;
	const char*		title=0;
	const char*		xFontName;
	const char*		xFontSize;
	const char*		xFontWeight;
	const char*		xFontSlant;
	const char*		xTabSize;
	UFreeType*		xFreeType;
	ostream*		oStream=0;
	ofstream*		oFStream=0;
#ifdef HAVE_STDIOSTREAM
	stdiostream*		oPStream=0;
#else
	ostdiostream*		oPStream=0;
#endif
	const char*		fontFile=0;
	UFreeType::UWeight	weight;
	UFreeType::USlant	slant;
	UPrint*			xPrint;
	UCS2*			header;
	int			headerSize;
	UPrint::UStatus		status;
	char			buffer[256];
	unsigned int		i;
	FILE*			file=0;

	layout->activate ();

	// Layout bug
	if (uLabel->sizeHint().width() > minimumSize().width())
	{
		resize (uLabel->sizeHint().width()+30, 
			minimumSize().height()+10);
	}
	else
	{
		resize (minimumSize().width()+10, minimumSize().height()+10);
	}

	message = (const char*) 0;
	if (topConfig->getPrinter()==0)
	{
		return OK;
	}
	if (fontCount==0)
	{
		message = Translate ("No True Type Fonts Defined.");
		return OK;
	}
	if (textSizeIn==0 || textIn==0 || (textSizeIn==1 && textSizesIn[0] == 0))
	{
		message = Translate ("Nothing to print.");
		return OK;
	}
	title =  (titleIn==0) ? "Untitled" : titleIn;
	lastButton = CANCEL;
	
	uLabel->setText (title);
	uSelectPrinter();
	if (!exec())
	{
		lastButton=CANCEL;
		return CANCEL;
	}
	lastButton=OK;
	printerName = uGetPrinter();
	if (printerName==0)
	{
		message = "Error: Internal Error";
		return OK;
	}
	printerCommand = topConfig->getPrinterCommand (printerName);
	if (printerCommand==0) return OK;

	xFontName = uGetItem (Config::FONTNAME);
	xFontSize = uGetItem (Config::FONTSIZE);
	xFontWeight = uGetItem (Config::FONTWEIGHT);
	xFontSlant = uGetItem (Config::FONTSLANT);
	xTabSize = uGetItem (Config::TABSIZE);
	if (xFontName==0 || xFontSize==0 || xFontWeight==0 
		|| xFontSlant==0 || xTabSize==0)
	{
		message = "Error: Internal Error";
		return OK;
	}
	weight = (strcmp (xFontWeight, "Bold")==0) ? UFreeType::BOLD: UFreeType::MEDIUM;
	slant = (strcmp (xFontSlant, "Italic")==0) ? UFreeType::ITALIC: UFreeType::ROMAN;

	xFreeType = UGetFreeTypePrint (xFontName);
	if (xFreeType==0) return OK;
	
	fontFile = xFreeType->getFileName (weight, slant);
	if (fontFile==0) return OK;

	if (printerCommand[0] == '|')
	{
		file = popen (&printerCommand[1], "w");
		if (file==0)
		{
			message = "Error: can not execute '";
			message += &printerCommand[1] ;
			message +=  "'.";
			return OK;
		}
#ifdef HAVE_STDIOSTREAM
		oPStream = new stdiostream (file);
#else
		oPStream = new ostdiostream (file);
#endif
		if (oPStream==0) return OK;

		oStream = new iostream (oPStream->rdbuf());
	}
	else
	{
		oFStream =  new ofstream (printerCommand);
		if (oFStream==0) return OK;
		if (oFStream->rdbuf() == 0 || !oFStream->rdbuf()->is_open())
		{
			message = "Can not open file: '";
			message += printerCommand;
			message += "'.";
			return OK;
		}
		oStream = oFStream;
	}

	// Finally we can start printing...
	xPrint = new UPrint (fontFile, atof (xFontSize), atoi (xTabSize)); 
	
	header = 0;
	headerSize = 0;
	if (titleIn !=0)
	{
		headerSize = strlen (titleIn);
		header = new UCS2[headerSize + 1];
		for (i=0; i<headerSize; i++)
		{
			header[i] = titleIn[i];
		}
		header[i] = 0;
	}

	status = xPrint->print (oStream, textIn, textSizesIn, textSizeIn, header, headerSize);

	if (header!=0)
	{
		delete header;
		header = 0;
	}
	

	if (status!=UPrint::OK)
	{
		message = (const char*) xPrint->errors;
	}
	else
	{
		sprintf (buffer, Translate ("Printed %d page(s)"), 
			xPrint->getPageCount());

		message = (const char*) buffer;
		if (xPrint->getWrappedCount() > 0)
		{
			message += "\n";
			sprintf (buffer, Translate ("Wrapped %d line(s)"), 
				xPrint->getWrappedCount());
			message += (const char*) buffer;
		}
	}
	delete xPrint;

	if (printerCommand[0] == '|')
	{
		delete oStream;
		delete oPStream;
		pclose (file);
	}
	else
	{
		oFStream->close();
		delete oFStream;
	}
	
	return OK;
}

const char*
PrintDialog::getMessage ()
{
	return (const char*) message;
}
void
PrintDialog::uAddList (QComboBox* box, const AStringList& list, 
	const char* item)
{
	int		i;

	for (i=0; i<list.getSize(); i++)
	{
		box->insertItem (list.at(i));
		if (i==0)
		{
			box->setCurrentItem (0);
		}
		if (strcmp (list.at(i), item)==0)
		{
			box->setCurrentItem (i);
		}
	}
	box->setMaximumSize (box->width(), box->height());
	box->setMinimumSize (0, box->height());
	return;
}

void
PrintDialog::uSelectPrinter()
{		
	int	i;

	for (i=0; i<topConfig->getPrinterList().getSize(); i++)
	{
		if (strcmp (topConfig->getPrinterList().at(i), 
			topConfig->getPrinter()) ==0)
		{
			uCheckPrinter->setCurrentItem (i);
		}
	}
}

const char*
PrintDialog::uGetPrinter()
{
	const char* newPrinter;
	newPrinter =  topConfig->getPrinterList().at(uCheckPrinter->currentItem());
	topConfig->setPrinter (newPrinter);
	return newPrinter;
}

const char*
PrintDialog::uGetItem (Config::CheckList index)
{
	switch (index)
	{
	case Config::FONTNAME:
		return uFontList.at (uCheckFontName->currentItem());

	case Config::FONTSIZE:
		return topConfig->getCheckList(index).at(uCheckFontSize->currentItem());
	case Config::FONTWEIGHT:
		return uFontWeightList.at(uCheckFontWeight->currentItem());

	case Config::FONTSLANT:
		return uFontSlantList.at(uCheckFontSlant->currentItem());

	case Config::TABSIZE:
		return topConfig->getCheckList(index).at(uCheckTabSize->currentItem());
	default:
		break;
	}
	// Never happens.
	return 0;
}
